@@ -17,6 +17,7 @@ from .request import urlopen
 
 GPGStore = gpg.GPGStore
 extract_pub_key_ids = gpg.extract_key_id
+get_default_gpg_config_dir = gpg.get_default_gpg_config_dir
 
 
 def to_user_error(error_types: Union[Exception, Tuple[Exception, ...]]):
@@ -86,16 +87,27 @@ def download_keys(fingerprints: List[str], keyserver: str,
         raise KeyserverError(action='download', keyserver=keyserver)
 
 
-def detach_sign_file(src: Union[bytes, io.IOBase], passphrase: str,
-                     gpg_store: GPGStore) -> bytes:
+def detach_sign_file(src: Union[bytes, io.IOBase], signature_fingerprint: str,
+                     passphrase: str, gpg_store: GPGStore) -> bytes:
     """Sign a file with a detached signature"""
     try:
-        with gpg_store.detach_sig(src, passphrase) as out:
+        with gpg_store.detach_sig(src, passphrase,
+                                  signature_fingerprint) as out:
             return out.read()
     except gpg.cmd.GPGError as e:
         raise UserError(f"File signing failed. {e}")
 
 
+def check_password(user_fingerprint: str, passphrase: str,
+                   gpg_store: GPGStore):
+    """Check if key password is correct"""
+    try:
+        with gpg_store.detach_sig(b"test", passphrase, user_fingerprint):
+            pass
+    except gpg.cmd.GPGError:
+        raise UserError("GPG password is incorrect")
+
+
 def verify_signature(src: Union[bytes, io.IOBase], sig: bytes,
                      gpg_store: GPGStore) -> str:
     """Verify a file with a detached signature.
