@@ -11,8 +11,7 @@ from PySide2 import QtCore, QtGui, QtWidgets
 from PySide2.QtCore import QSysInfo
 from PySide2.QtGui import QKeySequence
 
-from .component import ConsoleWidget, GuiProgress, FileSelectionWidget,\
-                       PathInput
+from .component import GuiProgress, FileSelectionWidget, PathInput, TabMixin
 from .parallel import Worker
 from .model import AppData, TableModel
 from .. import __version__, APP_NAME_SHORT, APP_NAME_LONG, protocols
@@ -27,11 +26,6 @@ Key = crypt.gpg.Key
 QtCore.QThreadPool.globalInstance().setExpiryTimeout(-1)
 
 
-class TabMixin:
-    def create_console(self):
-        self.console = ConsoleWidget("Console", self)
-
-
 class MainWindow(QtWidgets.QMainWindow):
     def __init__(self):
         super().__init__()
@@ -122,9 +116,10 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         files_panel = self.create_files_panel()
         user_panel = self.create_user_panel()
         output_panel = self.create_output_panel()
-        run_panel = self.create_run_panel()
-        self.progress_bar = QtWidgets.QProgressBar(self)
+        self.create_run_panel("Package and Encrypt data", self.encrypt,
+                              "Package && Encrypt")
         self.create_console()
+        self.create_progress_bar()
 
         layout = QtWidgets.QVBoxLayout()
         layout.addWidget(files_panel)
@@ -134,9 +129,9 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         layout_split.addWidget(output_panel)
         layout.addLayout(layout_split)
 
-        layout.addWidget(run_panel)
-        layout.addWidget(self.progress_bar)
+        layout.addWidget(self.run_panel)
         layout.addWidget(self.console)
+        layout.addWidget(self.progress_bar)
 
         self.setLayout(layout)
 
@@ -151,6 +146,8 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         box = QtWidgets.QGroupBox("Select data sender and recipients")
         sender_widget = QtWidgets.QComboBox()
         sender_widget.setModel(self.app_data.priv_keys_model)
+        if self.app_data.default_key_index:
+            sender_widget.setCurrentIndex(self.app_data.default_key_index)
 
         recipients_input_view = QtWidgets.QComboBox()
         recipients_input_view.setModel(self.app_data.pub_keys_model)
@@ -260,18 +257,6 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         box.setLayout(layout)
         return box
 
-    def create_run_panel(self):
-        box = QtWidgets.QGroupBox("Package and Encrypt data")
-        layout = QtWidgets.QHBoxLayout()
-        btn_test = QtWidgets.QPushButton("Test")
-        btn_test.clicked.connect(partial(self.encrypt, dry_run=True))
-        btn_encrypt = QtWidgets.QPushButton("Package && Encrypt")
-        btn_encrypt.clicked.connect(self.encrypt)
-        layout.addWidget(btn_test)
-        layout.addWidget(btn_encrypt)
-        box.setLayout(layout)
-        return box
-
     def encrypt(self, dry_run=False):
         warning_msg = []
         if not self.app_data.encrypt_files:
@@ -315,9 +300,7 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             compress=self.app_data.encrypt_compress,
             progress=progress,
             on_error=lambda: None)
-        worker.signals.error.connect(lambda e:
-                                     self.console.append(str(e[1])))
-        worker.signals.logging.connect(self.console.write)
+        self.add_worker_actions(worker)
         self.threadpool.start(worker)
 
 
@@ -329,16 +312,17 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
 
         files_panel = self.create_files_panel()
         decrypt_options_panel = self.create_decrypt_options_panel()
-        run_panel = self.create_run_panel()
-        self.progress_bar = QtWidgets.QProgressBar(self)
+        self.create_run_panel("Decrypt data", self.decrypt,
+                              "Decrypt selected files")
         self.create_console()
+        self.create_progress_bar()
 
         layout = QtWidgets.QGridLayout()
         layout.addWidget(files_panel, 0, 0, 1, 2)
         layout.addWidget(decrypt_options_panel, 1, 0, 1, 2)
-        layout.addWidget(run_panel, 2, 0, 1, 2)
-        layout.addWidget(self.progress_bar, 3, 0, 1, 2)
-        layout.addWidget(self.console, 4, 0, 1, 2)
+        layout.addWidget(self.run_panel, 2, 0, 1, 2)
+        layout.addWidget(self.console, 3, 0, 1, 2)
+        layout.addWidget(self.progress_bar, 4, 0, 1, 2)
         self.setLayout(layout)
 
     def create_files_panel(self):
@@ -375,18 +359,6 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
         box.setLayout(layout)
         return box
 
-    def create_run_panel(self):
-        box = QtWidgets.QGroupBox("Decrypt data")
-        layout = QtWidgets.QHBoxLayout()
-        btn_test = QtWidgets.QPushButton("Test")
-        btn_test.clicked.connect(partial(self.decrypt, dry_run=True))
-        btn_decrypt = QtWidgets.QPushButton("Decrypt selected files")
-        btn_decrypt.clicked.connect(self.decrypt)
-        layout.addWidget(btn_test)
-        layout.addWidget(btn_decrypt)
-        box.setLayout(layout)
-        return box
-
     def decrypt(self, dry_run=False):
         if not self.app_data.decrypt_files:
             msg_warn = QtWidgets.QMessageBox()
@@ -414,9 +386,7 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
             dry_run=dry_run,
             progress=progress,
             on_error=lambda: None)
-        worker.signals.error.connect(lambda e:
-                                    self.console.append(str(e[1])))
-        worker.signals.logging.connect(self.console.write)
+        self.add_worker_actions(worker)
         self.threadpool.start(worker)
 
 
@@ -430,16 +400,17 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
 
         files_panel = self.create_files_panel()
         options_panel = self.create_options_panel()
-        run_panel = self.create_run_panel()
-        self.progress_bar = QtWidgets.QProgressBar(self)
+        self.create_run_panel("Transfer data", self.transfer,
+                              "Transfer selected files")
         self.create_console()
+        self.create_progress_bar()
 
         layout = QtWidgets.QGridLayout()
         layout.addWidget(files_panel, 0, 0, 1, 2)
         layout.addWidget(options_panel, 1, 0, 1, 2)
-        layout.addWidget(run_panel, 2, 0, 1, 2)
-        layout.addWidget(self.progress_bar, 3, 0, 1, 2)
-        layout.addWidget(self.console, 4, 0, 1, 2)
+        layout.addWidget(self.run_panel, 2, 0, 1, 2)
+        layout.addWidget(self.console, 3, 0, 1, 2)
+        layout.addWidget(self.progress_bar, 4, 0, 1, 2)
         self.setLayout(layout)
 
     def create_files_panel(self):
@@ -563,18 +534,6 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         load_connection()
         return box
 
-    def create_run_panel(self):
-        box = QtWidgets.QGroupBox("Transfer data")
-        layout = QtWidgets.QHBoxLayout()
-        btn_test = QtWidgets.QPushButton("Test")
-        btn_test.clicked.connect(partial(self.transfer, dry_run=True))
-        btn_transfer = QtWidgets.QPushButton("Transfer selected files")
-        btn_transfer.clicked.connect(self.transfer)
-        layout.addWidget(btn_test)
-        layout.addWidget(btn_transfer)
-        box.setLayout(layout)
-        return box
-
     def transfer(self, dry_run=False):
         if not self.app_data.transfer_files:
             msg_warn = QtWidgets.QMessageBox()
@@ -626,9 +585,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
             progress=progress,
             on_error=lambda: None,
             two_factor_callback=second_factor_callback)
-        worker.signals.error.connect(lambda e:
-                                     self.console.append(str(e[1])))
-        worker.signals.logging.connect(self.console.write)
+        self.add_worker_actions(worker)
         msg_signal.msg.connect(show_second_factor_dialog)
         self.threadpool.start(worker)
 
@@ -935,6 +892,14 @@ class KeysTab(QtWidgets.QWidget):
     def update_private_keys(self):
         # Retrieve all private keys from local keyring.
         keys_private = self.app_data.config.gpg_store.list_sec_keys(keys=())
+        try:
+            default_key = (self.app_data.config.default_sender or
+                           self.app_data.config.gpg_store.default_key())
+            self.app_data.default_key_index = next(index for index, entry in enumerate(
+                keys_private) if entry.fingerprint == default_key)
+            self.app_data.encrypt_sender = default_key
+        except StopIteration:
+            pass
         self.app_data.priv_keys_model.set_data(
             {f"{key.uids[0]} {key.key_id}": key for key in keys_private})
 
